@@ -41,6 +41,11 @@ from keystone.federation import utils as mapping_engine
 from keystone.i18n import _
 from keystone.server import backends
 
+# We need to define the log level to INFO. Otherwise, when using the
+# system, we will not be able to see anything.
+log.set_defaults(default_log_levels="INFO")
+
+
 CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
@@ -184,8 +189,10 @@ class BootStrap(BaseApp):
             self.bootstrapper.immutable_roles = True
 
         self.bootstrapper.bootstrap()
+        self.service_role_id = self.bootstrapper.service_role_id
         self.reader_role_id = self.bootstrapper.reader_role_id
         self.member_role_id = self.bootstrapper.member_role_id
+        self.manager_role_id = self.bootstrapper.manager_role_id
         self.role_id = self.bootstrapper.admin_role_id
         self.project_id = self.bootstrapper.project_id
 
@@ -354,20 +361,12 @@ class DbSync(BaseApp):
             )
             return 2
 
-        if isinstance(expand_version, int):
-            # we're still using sqlalchemy-migrate
-            LOG.info(
-                'Your database is currently using legacy version control. '
-                'Your first step is to run `keystone-manage db_sync --expand`.'
-            )
-            return 2
-
         try:
             contract_version = upgrades.get_db_version(branch='contract')
         except db_exception.DBMigrationError:
             contract_version = None
 
-        heads = upgrades.get_current_heads()
+        heads = upgrades.get_head_revisions()
 
         if (
             upgrades.EXPAND_BRANCH not in heads or
@@ -548,7 +547,7 @@ class FernetSetup(BasePermissionsSetup):
                 os.path.abspath(CONF.fernet_receipts.key_repository)):
             cls.initialize_fernet_repository(
                 keystone_user_id, keystone_group_id, 'fernet_receipts')
-        elif(CONF.fernet_tokens.max_active_keys !=
+        elif (CONF.fernet_tokens.max_active_keys !=
                 CONF.fernet_receipts.max_active_keys):
             # WARNING(adriant): If the directories are the same,
             # 'max_active_keys' is ignored from fernet_receipts in favor of
@@ -1259,6 +1258,8 @@ class MappingEngineTester(BaseApp):
             raise SystemExit(_("Error while opening file "
                                "%(path)s: %(err)s") % {'path': path, 'err': e})
 
+        LOG.debug("Assertions loaded: [%s].", self.assertion)
+
     def normalize_assertion(self):
         def split(line, line_num):
             try:
@@ -1294,6 +1295,9 @@ class MappingEngineTester(BaseApp):
     def main(cls):
         if CONF.command.engine_debug:
             mapping_engine.LOG.logger.setLevel('DEBUG')
+            LOG.logger.setLevel('DEBUG')
+
+            LOG.debug("Debug log level enabled!")
         else:
             mapping_engine.LOG.logger.setLevel('WARN')
 
@@ -1301,7 +1305,23 @@ class MappingEngineTester(BaseApp):
 
         tester.read_rules(CONF.command.rules)
         tester.normalize_rules()
-        mapping_engine.validate_mapping_structure(tester.rules)
+
+        attribute_mapping = tester.rules.copy()
+        if CONF.command.mapping_schema_version:
+            attribute_mapping[
+                'schema_version'] = CONF.command.mapping_schema_version
+
+        if not attribute_mapping.get('schema_version'):
+            default_schema_version = '1.0'
+            LOG.warning('No schema version defined in rules [%s]. Therefore,'
+                        'we will use the default as [%s].', attribute_mapping,
+                        default_schema_version)
+            attribute_mapping[
+                'schema_version'] = default_schema_version
+
+        LOG.info("Validating Attribute mapping rules [%s].", attribute_mapping)
+        mapping_engine.validate_mapping_structure(attribute_mapping)
+        LOG.info("Attribute mapping rules are valid.")
 
         tester.read_assertion(CONF.command.input)
         tester.normalize_assertion()
@@ -1315,6 +1335,8 @@ class MappingEngineTester(BaseApp):
         rp = mapping_engine.RuleProcessor(tester.mapping_id,
                                           tester.rules['rules'])
         mapped = rp.process(tester.assertion)
+
+        LOG.info("Result of the attribute mapping processing.")
         print(jsonutils.dumps(mapped, indent=2))
 
     @classmethod
@@ -1349,6 +1371,11 @@ class MappingEngineTester(BaseApp):
                             default=False, action="store_true",
                             help=("Enable debug messages from the mapping "
                                   "engine."))
+        parser.add_argument('--mapping-schema-version', default=None,
+                            required=False,
+                            help=("The override for the schema version of "
+                                  "the rules that are loaded in the 'rules' "
+                                  "option of the test CLI."))
 
 
 class MappingPopulate(BaseApp):
